# Epidemic model: discrete-time, chain binomial SIR model
#
# Based on:
#   Author: Nele Goeyvaerts
#   Last update: 11/06/2014
# 
# Last update: 09/05/2016
######################################################

############################################################

## Model versions:
#	1. Chain binomial model with constant recovery probability

## Note: with week/weekend distinction, a time step is considered to be 1 day

## Input parameters:
#	HH_network: static network object used for epidemic simulation (edge = event by which transmission may occur)
#	beta1: within-household transmission probability per contact (link), per time step
#	p_contact: within-household probability of contact (differs from 1 only for random mixing)
#	beta2: community transmission probability per contact (link), per time step (a constant or a child by adult 2x2 matrix)
#	gamma: parameter(s) specifying the duration of the infectious period
# 	nSeed: number of vertices that are seeded as infectious individuals
#	HH_network_weekend (optional): static network object. If supplied, a distinction is made between weekdays (HH_network) and weekends.
# p_contact_weekend (optional): within-household probability of contact (differs from 1 only for random mixing). If supplied, a distinction is made between weekdays and weekends.
#		Note: the vertex list of HH_network is used as a reference!

## Output parameters:
#	timeSteps: duration of the epidemic (number of time steps)
#	infectedMat: matrix indicating the current state (columns) for every vertex (rows) in the network: suceptible (0), infected (1) and recovered (NA)
#	incidenceMat: matrix indicating whether a new infection occurred (rows) at each time step (columns) 
#	beta2: community transmission probability per contact (link), per time step (a constant or a child by adult 2x2 matrix)
#	HH_network: static network object used for epidemic simulation (edge = event by which transmission may occur)
#	weekend: indicator vector of weekend (=1) or weekday (=0), for each simulated time step


epiChainBinModel1 <- function(HH_network, beta1, p_contact, beta2, gamma, nSeed, HH_network_weekend, p_contact_weekend){

	# Infectious period assumption: basic constant recovery probability per time step
	# Parameters:
	#		gamma: recovery probability per time step

	# Initial values
	beta2 <- matrix(beta2, nrow = 2, ncol = 2)
	timeSteps <- 1
	if (!hasArg(HH_network_weekend)){
		HH_network_weekend <- HH_network	# copy regular networks when week/weekend distinction is not required
	}
	if (!hasArg(p_contact_weekend)){
	  p_contact_weekend <- p_contact	# copy contact prob when week/weekend distinction is not required
	}	
  weekend <- NULL
	draw <- sample(0:6, 1)				# randomly allocate weekend numbers (to shuffle the first day of epidemic simulation)
	weekendNrs <- c(draw, (draw+1) %% 7)
	nrVertices <- network.size(HH_network)
	infected <- rep(0, nrVertices)
	infected[sample(1:nrVertices, nSeed)] <- 1		
	infectedMat <- infected
	incidenceMat <- infected
  hhfoiMat <- NULL
  comfoiMat_child <- NULL
	comfoiMat_adult <- NULL
	
	# Individual attributes
	hh_id <- HH_network %v% "hh_id"
	vertexNames <- HH_network %v% "vertex.names"
	age <- HH_network %v% "age"
	child <- age <= 18	# Note: this is not equal to the "child" vertex attribute! Some children are older than 18 years
	vertex_id <- 1:nrVertices

	while(sum(infected, na.rm = TRUE) > 0){ 
		# Repeat loop until all infected individuals are recovered and no new infections have occurred
	  hhfoi <- rep(NA, nrVertices)
	  comfoi_child <- rep(NA, nrVertices) 
	  comfoi_adult <- rep(NA, nrVertices)      
		# Infect each susceptible with a certain probability
		incidence <- rep(0, nrVertices)
		infectedPrev <- infected	# indicator of infection in the previous time step
		for(i in vertex_id[!is.na(infected) & infected == 0]){
			if ((timeSteps %% 7) %in% weekendNrs){
				weekend[timeSteps] <- 1
				hh_contacts <- get.neighborhood(HH_network_weekend, i)
        prob.c = p_contact_weekend
				}else{
				weekend[timeSteps] <- 0
				hh_contacts <- get.neighborhood(HH_network, i)
        prob.c = p_contact
				}	
			nrInfHhChild <- sum(infectedPrev[vertex_id %in% hh_contacts & child == TRUE], na.rm = TRUE)
			nrInfHhAdult <- sum(infectedPrev[vertex_id %in% hh_contacts & child == FALSE], na.rm = TRUE)
			nrInfHH <- nrInfHhChild + nrInfHhAdult
			nrInfComChild <- sum(infectedPrev[child == TRUE], na.rm = TRUE) - nrInfHhChild
			nrInfComAdult <- sum(infectedPrev[child == FALSE], na.rm = TRUE) - nrInfHhAdult
			if(child[i] == TRUE){
				infProb <- 1-(1-beta1*prob.c)^nrInfHH*(1-beta2[1,1])^nrInfComChild*(1-beta2[1,2])^nrInfComAdult
				comfoi_child[i] <- beta2[1,1]*nrInfComChild; comfoi_adult[i] <- beta2[1,2]*nrInfComAdult
				}else{
				infProb <- 1-(1-beta1*prob.c)^nrInfHH*(1-beta2[2,2])^nrInfComAdult*(1-beta2[1,2])^nrInfComChild
				comfoi_child[i] <- beta2[1,2]*nrInfComChild; comfoi_adult[i] <- beta2[2,2]*nrInfComAdult
				}
			infected[i] <- ifelse(runif(1) < infProb, 1, 0)
			incidence[i] <- infected[i]
      hhfoi[i] <- beta1*prob.c*nrInfHH
			}

		# Recover each infected with a certain probability
		for(i in vertexNames[!is.na(infected) & infected == 1]){
			infected[vertexNames == i] <- ifelse(runif(1) < gamma, NA, 1)
			}

		# Append current status to matrix and update time variable
		infectedMat <- cbind(infectedMat, infected)
		incidenceMat <- cbind(incidenceMat, incidence)
    hhfoiMat <- cbind(hhfoiMat, hhfoi)
    comfoiMat_child <- cbind(comfoiMat_child, comfoi_child)
    comfoiMat_adult <- cbind(comfoiMat_adult, comfoi_adult)
    timeSteps <- timeSteps + 1
		print(sum(infected, na.rm = TRUE))

		}
	# List of output variables
	return(list(timeSteps = timeSteps, infectedMat = infectedMat, incidenceMat = incidenceMat, hhfoiMat = hhfoiMat, comfoiMat_child = comfoiMat_child, comfoiMat_adult = comfoiMat_adult, beta2 = beta2, HH_network = HH_network, weekend = weekend))
	}

## hh transmission children/adults
epiChainBinModel11 <- function(HH_network, beta1, p_contact, beta2, gamma, nSeed, HH_network_weekend, p_contact_weekend){
  
  # Infectious period assumption: basic constant recovery probability per time step
  # Parameters:
  #		gamma: recovery probability per time step
  
  # Initial values
  beta2 <- matrix(beta2, nrow = 2, ncol = 2)
  timeSteps <- 1
  if (!hasArg(HH_network_weekend)){
    HH_network_weekend <- HH_network	# copy regular networks when week/weekend distinction is not required
  }
  if (!hasArg(p_contact_weekend)){
    p_contact_weekend <- p_contact	# copy contact prob when week/weekend distinction is not required
  }	
  weekend <- NULL
  draw <- sample(0:6, 1)				# randomly allocate weekend numbers (to shuffle the first day of epidemic simulation)
  weekendNrs <- c(draw, (draw+1) %% 7)
  nrVertices <- network.size(HH_network)
  infected <- rep(0, nrVertices)
  infected[sample(1:nrVertices, nSeed)] <- 1		
  infectedMat <- infected
  incidenceMat <- infected
  hhfoiMat_child <- NULL
  hhfoiMat_adult <- NULL
  comfoiMat_child <- NULL
  comfoiMat_adult <- NULL
  
  # Individual attributes
  hh_id <- HH_network %v% "hh_id"
  vertexNames <- HH_network %v% "vertex.names"
  age <- HH_network %v% "age"
  child <- age <= 18	# Note: this is not equal to the "child" vertex attribute! Some children are older than 18 years
  vertex_id <- 1:nrVertices
  
  while(sum(infected, na.rm = TRUE) > 0){ 
    # Repeat loop until all infected individuals are recovered and no new infections have occurred
    hhfoi_child <- rep(NA, nrVertices)
    hhfoi_adult <- rep(NA, nrVertices)
    comfoi_child <- rep(NA, nrVertices) 
    comfoi_adult <- rep(NA, nrVertices)      
    # Infect each susceptible with a certain probability
    incidence <- rep(0, nrVertices)
    infectedPrev <- infected	# indicator of infection in the previous time step
    for(i in vertex_id[!is.na(infected) & infected == 0]){
      if ((timeSteps %% 7) %in% weekendNrs){
        weekend[timeSteps] <- 1
        hh_contacts <- get.neighborhood(HH_network_weekend, i)
        prob.c = p_contact_weekend
      }else{
        weekend[timeSteps] <- 0
        hh_contacts <- get.neighborhood(HH_network, i)
        prob.c = p_contact
      }	
      nrInfHhChild <- sum(infectedPrev[vertex_id %in% hh_contacts & child == TRUE], na.rm = TRUE)
      nrInfHhAdult <- sum(infectedPrev[vertex_id %in% hh_contacts & child == FALSE], na.rm = TRUE)
      nrInfHH <- nrInfHhChild + nrInfHhAdult
      nrInfComChild <- sum(infectedPrev[child == TRUE], na.rm = TRUE) - nrInfHhChild
      nrInfComAdult <- sum(infectedPrev[child == FALSE], na.rm = TRUE) - nrInfHhAdult
      if(child[i] == TRUE){
        infProb <- 1-(1-beta1[1]*prob.c)^nrInfHhChild*(1-beta1[2]*prob.c)^nrInfHhAdult*(1-beta2[1,1])^nrInfComChild*(1-beta2[1,2])^nrInfComAdult
        comfoi_child[i] <- beta2[1,1]*nrInfComChild; comfoi_adult[i] <- beta2[1,2]*nrInfComAdult
      }else{
        infProb <- 1-(1-beta1[1]*prob.c)^nrInfHhChild*(1-beta1[2]*prob.c)^nrInfHhAdult*(1-beta2[2,2])^nrInfComAdult*(1-beta2[1,2])^nrInfComChild
        comfoi_child[i] <- beta2[1,2]*nrInfComChild; comfoi_adult[i] <- beta2[2,2]*nrInfComAdult
      }
      infected[i] <- ifelse(runif(1) < infProb, 1, 0)
      incidence[i] <- infected[i]
      hhfoi_child[i] <- beta1[1]*prob.c*nrInfHhChild
      hhfoi_adult[i] <- beta1[2]*prob.c*nrInfHhAdult
    }
    
    # Recover each infected with a certain probability
    for(i in vertexNames[!is.na(infected) & infected == 1]){
      infected[vertexNames == i] <- ifelse(runif(1) < gamma, NA, 1)
    }
    
    # Append current status to matrix and update time variable
    infectedMat <- cbind(infectedMat, infected)
    incidenceMat <- cbind(incidenceMat, incidence)
    hhfoiMat_child <- cbind(hhfoiMat_child, hhfoi_child)
    hhfoiMat_adult <- cbind(hhfoiMat_adult, hhfoi_adult)
    comfoiMat_child <- cbind(comfoiMat_child, comfoi_child)
    comfoiMat_adult <- cbind(comfoiMat_adult, comfoi_adult)
    timeSteps <- timeSteps + 1
    print(sum(infected, na.rm = TRUE))
    
  }
  # List of output variables
  return(list(timeSteps = timeSteps, infectedMat = infectedMat, incidenceMat = incidenceMat, hhfoiMat_child = hhfoiMat_child, hhfoiMat_adult = hhfoiMat_adult, comfoiMat_child = comfoiMat_child, comfoiMat_adult = comfoiMat_adult, beta2 = beta2, HH_network = HH_network, weekend = weekend))
}


## Constant community foi: beta2  
epiChainBinModel2 <- function(HH_network, beta1, p_contact, beta2, gamma, nSeed, HH_network_weekend, p_contact_weekend){
  
  # Infectious period assumption: basic constant recovery probability per time step
  # Parameters:
  #  	gamma: recovery probability per time step
  
  # Initial values
  timeSteps <- 1
  if (!hasArg(HH_network_weekend)){
    HH_network_weekend <- HH_network	# copy regular networks when week/weekend distinction is not required
  }
  if (!hasArg(p_contact_weekend)){
    p_contact_weekend <- p_contact	# copy contact prob when week/weekend distinction is not required
  }	
  weekend <- NULL
  draw <- sample(0:6, 1)				# randomly allocate weekend numbers (to shuffle the first day of epidemic simulation)
  weekendNrs <- c(draw, (draw+1) %% 7)
  nrVertices <- network.size(HH_network)
  infected <- rep(0, nrVertices)
  infected[sample(1:nrVertices, nSeed)] <- 1		
  infectedMat <- infected
  incidenceMat <- infected
  hhfoiMat <- NULL
  
  # Individual attributes
  hh_id <- HH_network %v% "hh_id"
  vertexNames <- HH_network %v% "vertex.names"
  age <- HH_network %v% "age"
  child <- age <= 18	# Note: this is not equal to the "child" vertex attribute! Some children are older than 18 years
  vertex_id <- 1:nrVertices
  
  while(sum(infected, na.rm = TRUE) > 0){ 
    # Repeat loop until all infected individuals are recovered and no new infections have occurred
    hhfoi <- rep(NA, nrVertices)
    comfoi_child <- rep(NA, nrVertices) 
    comfoi_adult <- rep(NA, nrVertices)  
    # Infect each susceptible with a certain probability
    incidence <- rep(0, nrVertices)
    infectedPrev <- infected	# indicator of infection in the previous time step
    for(i in vertex_id[!is.na(infected) & infected == 0]){
      if ((timeSteps %% 7) %in% weekendNrs){
        weekend[timeSteps] <- 1
        hh_contacts <- get.neighborhood(HH_network_weekend, i)
        prob.c = p_contact_weekend
      }else{
        weekend[timeSteps] <- 0
        hh_contacts <- get.neighborhood(HH_network, i)
        prob.c = p_contact
      }	
      nrInfHhChild <- sum(infectedPrev[vertex_id %in% hh_contacts & child == TRUE], na.rm = TRUE)
      nrInfHhAdult <- sum(infectedPrev[vertex_id %in% hh_contacts & child == FALSE], na.rm = TRUE)
      nrInfHH <- nrInfHhChild + nrInfHhAdult
      if(child[i] == TRUE){
        infProb <- 1-(1-beta1*prob.c)^nrInfHH*(1-beta2)
      }else{
        infProb <- 1-(1-beta1*prob.c)^nrInfHH*(1-beta2)
      }
      infected[i] <- ifelse(runif(1) < infProb, 1, 0)
      incidence[i] <- infected[i]
      hhfoi[i] <- beta1*prob.c*nrInfHH
    }
    
    # Recover each infected with a certain probability
    for(i in vertexNames[!is.na(infected) & infected == 1]){
      infected[vertexNames == i] <- ifelse(runif(1) < gamma, NA, 1)
    }
    
    # Append current status to matrix and update time variable
    infectedMat <- cbind(infectedMat, infected)
    incidenceMat <- cbind(incidenceMat, incidence)
    hhfoiMat <- cbind(hhfoiMat, hhfoi)
    timeSteps <- timeSteps + 1
    print(sum(infected, na.rm = TRUE))
    
  }
  # List of output variables
  return(list(timeSteps = timeSteps, infectedMat = infectedMat, incidenceMat = incidenceMat, hhfoiMat = hhfoiMat, beta2 = beta2, HH_network = HH_network, weekend = weekend))
}

## Independent households: no community transmission
epiChainBinModel_ind <- function(HH_network, beta1, p_contact, gamma, HH_network_weekend, p_contact_weekend){
  
  ## Input parameters:
  # HH_network: static network object (edge = event by which transmission may occur) 
  # betaHH: within-household transmission probability per contact (link), per time step
  # gamma: recovery probability per time step
  ## Output parameters:
  #	timeSteps: duration of the epidemic (number of time steps)
  #	statusMat: matrix indicating the current state (columns) for every vertex (rows) in the network: suceptible (0), exposed (1), infectious (2) and recovered (-1)
  #	incidenceMat: matrix indicating whether a new infection (= 1) occurred (rows) at each time step (columns) 
  #	HH_network: static network object (edge = event by which transmission may occur)
  
  betaHH = beta1
  if (!hasArg(HH_network_weekend)){
    HH_network_weekend <- HH_network	
  }
  if (!hasArg(p_contact_weekend)){
    p_contact_weekend <- p_contact	
  }	
    
  # Attributes
  nrVertices <- network.size(HH_network)
  hh_id <- HH_network %v% "hh_id"
  hh_id_unique <- unique(hh_id)
  nrHH <- length(hh_id_unique)
  vertexNames <- HH_network %v% "vertex.names"
  vertex_id <- 1:nrVertices
  
  ## Incidence and status matrices
  incidenceMat = NULL
  statusMat = NULL
  weekend <- NULL
  
  for (j in 1:nrHH){
    
    hh_select = hh_id==hh_id_unique[j]
    vertexNames_hh = vertexNames[hh_select]
    vertex_id_hh = vertex_id[hh_select]
    
    # Initial exposed individual in HH
    status <- rep(0, sum(hh_select))
    update.select <- sample(c(1:sum(hh_select)),1)
    status[update.select] <- 1
    
    # Primary case randomly in week or weekend
    draw <- sample(0:6, 1)				
    weekendNrs <- c(draw, (draw+1) %% 7)
    
    # Incidence and status matrices
    incidenceMat[[j]] = as.matrix(status)
    statusMat[[j]] = as.matrix(status)
    
    timeSteps <- 1
    
    while(sum(status==1)>0){ 
      # Repeat loop until all infected individuals are recovered and no new infections have occurred in the HH
      
      statusPrev <- status  # indicator of infection status in the previous time step
      incidence <- rep(0, sum(hh_select))
      
      # Infect each susceptible with a certain probability
      for(i in vertexNames_hh[statusPrev == 0]){
        select <- vertexNames_hh == i
        if ((timeSteps %% 7) %in% weekendNrs){
          hh_contacts <- get.neighborhood(HH_network_weekend, vertex_id_hh[select])
          prob.c = p_contact_weekend
          weekend[timeSteps] <- 1
        }else{
          hh_contacts <- get.neighborhood(HH_network, vertex_id_hh[select])
          prob.c = p_contact
          weekend[timeSteps] <- 0
        }
        nrInfHH <- sum(statusPrev[vertex_id_hh %in% hh_contacts]==1)
        infProb <- 1-(1-betaHH*prob.c)^nrInfHH
        status[select] <- ifelse(runif(1) < infProb, 1, 0)
        incidence[select] <- status[select]
      }
      
  
      # Recover each infectious individual with a certain probability
      for(i in vertexNames_hh[statusPrev == 1]){
          status[vertexNames_hh == i] <- ifelse(runif(1) < gamma, -1, 1)
      }
      
      # Append current status to matrix and update time variable
      statusMat[[j]] <- cbind(statusMat[[j]],status)
      incidenceMat[[j]] <- cbind(incidenceMat[[j]],incidence)
      timeSteps <- timeSteps + 1
      #print(c(sum(status==1),sum(status==2))) 
    }
  }
  # List of output variables
  return(list(timeSteps = timeSteps, statusMat = statusMat, incidenceMat = incidenceMat, HH_network = HH_network, weekend = weekend))
}

