# Epidemic model: functions to plot the prevalence and incidence 
#
# Author: Nele Goeyvaerts
#
# Last update: 28/02/2014
######################################################################


## Function to plot summary statistics of the epidemic

plotSummary <- function(statistics){
	#	statistics: output from the epiSummary function

	par(mfrow = c(2,2))
	hist(statistics$infPeriod, main = "", xlab = "Infectious period", ylab = "Number of individuals")
	hist(statistics$hhPropInfected, main = "", xlab = "Proportion infected per HH", ylab = "Number of households")
	hist(statistics$hhDuration, main = "", xlab = "Time first onset to last case", ylab = "Number of households")
	boxplot(statistics$hhDuration ~ statistics$hhSize, xlab = "HH size", ylab = "Time first onset to last case")
	}


## Function to plot the prevalence for each S-I-R state (total or fraction)

plotPrevalence <- function(result, popfrac, transparency){
	# 	result: output from the epiChainBinModel function
	#	popfrac: plot absolute numbers (popfrac = FALSE) or proportions (popfrac = TRUE)
	#	transparency: color semi-transparency

	status <- result$infectedMat
	N <- dim(status)[1]
	nrI <- apply(status, 2, sum, na.rm = TRUE)
	nrR <- apply(is.na(status), 2, sum, na.rm = TRUE)
	nrS <- N - nrI - nrR
	# plot absolute numbers or fractions
	colors <- c("steelblue", "firebrick", "seagreen")
	width <- 3
	if (popfrac == FALSE){
		plot(nrS, type = "l", ylim = c(0, N), xlab = "Time", ylab = "Number of individuals", cex.lab = 1.2
			, col = alpha(colors[1], transparency), lwd = width, bty = "l")
		lines(nrI, col = alpha(colors[2], transparency), lwd = width)
		lines(nrR, col = alpha(colors[3], transparency), lwd = width)
		legend("right", c("S", "I", "R"), col = alpha(colors, transparency), lwd = width)
	}else{
		plot(nrS/N, type = "l", ylim = c(0, 1), xlab = "Time", ylab = "Proportion of individuals", cex.lab = 1.2
			, col = alpha(colors[1], transparency), lwd = width, bty = "l")
		lines(nrI/N, col = alpha(colors[2], transparency), lwd = width)
		lines(nrR/N, col = alpha(colors[3], transparency), lwd = width)
		legend("right", c("S", "I", "R"), col = alpha(colors, transparency), lwd = width)
	}
}


## Function to plot the disease incidence i.e. newly infected cases (total or fraction)

plotIncidence <- function(result, popfrac, color, transparency){
	# 	result: output from the epiChainBinModel function
	#	popfrac: plot absolute numbers (popfrac = FALSE) or proportions (popfrac = TRUE)
	#	transparency: color semi-transparency

	incidence <- result$incidenceMat
	N <- dim(incidence)[1]
	HH_network <- result$HH_network
	child <- HH_network %v% "child"
	newInf <- apply(incidence, 2, sum, na.rm = TRUE)
	newInf_ch <- apply(incidence[child == TRUE,], 2, sum, na.rm = TRUE)
	newInf_ad <- apply(incidence[child == FALSE,], 2, sum, na.rm = TRUE)

	# plot absolute numbers or fractions
	width <- 3
	if (popfrac == FALSE){
		plot(newInf, type = "l", ylim = c(0, max(newInf)), xlab = "Time", ylab = "Total incidence", cex.lab = 1.2
			, col = alpha(color, transparency), lwd = width, bty = "l")
		lines(newInf_ch, lty = 2, col = alpha(color, transparency), lwd = width)
		lines(newInf_ad, lty = 3, col = alpha(color, transparency), lwd = width)
	}else{
		plot(newInf/N, type = "l", ylim = c(0, max(newInf/N)), xlab = "Time", ylab = "Incidence (fraction)", cex.lab = 1.2
			, col = alpha(color, transparency), lwd = width, bty = "l")
		lines(newInf_ch/N, lty = 2, col = alpha(color, transparency), lwd = width)
		lines(newInf_ad/N, lty = 3, col = alpha(color, transparency), lwd = width)
	}
	legend("right", legend = c("Total", "Children", "Parents"), lty = 1:3, col = alpha(color, transparency), lwd = width)
}


## Function to combine and plot the disease incidence resulting from different simulations

plotSimIncidence <- function(incidence, timeSteps, sampleSize, color, transparency, width, main, ylab, ylim){
	# 	incidence: matrix containing incidence over time (columns) per simulation (rows)
	# 	timeSteps: number of time steps considered
	# 	sampleSize: number of simulation results displayed (random sample)
	#	color: color used for plotting
	#	transparency: color semi-transparency
	#	width: line width
	# 	main: plot title
	#	ylab: y-axis label
	#	ylim: limits y-axis

	nSim <- dim(incidence)[1]
	meanIncidence <- apply(incidence, 2, mean,na.rm=T)
	quantilesIncidence <- apply(incidence, 2, quantile, probs = c(0.1, 0.9),na.rm=T)
	incidence <- incidence[sample(1:nSim, sampleSize), 1:timeSteps]
	plot(incidence[1, ], type = "l", ylim = ylim, xlab = "Time", ylab = ylab, col = alpha(color, transparency)
			, lwd = width, bty = "l", main = main, cex.main = 1)
	for (i in 2:sampleSize){
		lines(incidence[i, ], col = alpha(color, transparency), lwd = width)
		}
	lines(meanIncidence, lwd = 2)
	#lines(quantilesIncidence[1,], lty = 2, lwd = 1.5)
	#lines(quantilesIncidence[2,], lty = 2, lwd = 1.5)
	}


## Function to plot the current state of the network at the requested time points

plotNetwork <- function(result, timePoints, size, flashy, seed){
	# 	result: output from the epiChainBinModel function
	#	timePoints: vector of requested time points
	#	size: number of households that need to be plotted
	#	flashy: choose display type - flashy colors (TRUE) or not (FALSE)
	#	seed: seed to fix the outcome (random sample)
	
	# Current state of the network at the requested time points
	status <- result$infectedMat[, timePoints]
	nrTimePoints <- length(timePoints)

	# Take a random sample of households
	HH_network <- result$HH_network
	hh_id <- HH_network %v% "hh_id"
	set.seed(seed)
	sample_hh_id <- sample(unique(hh_id), size = size, replace = FALSE)
	
	# Initiate plot
	if (flashy == TRUE){
		colors <- c("blue", "red", "green")
		transparency <- 1
		vertex.border <- "black"
		edge.col <- "black"
	}else{
		colors <- c("steelblue", "firebrick", "seagreen")
		transparency <- 0.8
		vertex.border <- "lightgrey"
		edge.col <- "grey"
	}
	par(mfrow = c(ceiling(nrTimePoints/2), min(c(2, nrTimePoints)) ))
	for (i in 1:nrTimePoints){
		# Add individual states as vertex attribute to the network
		state <- as.matrix(status)[,i]
		state[is.na(state)] <- 3	# R: NA -> 3
		state[state == 1] <- 2		# I: 1 -> 2
		state[state == 0] <- 1		# S: 0 -> 1
		set.seed(seed)
		plot(HH_network %s% which(hh_id %in% sample_hh_id), main = paste("Status at time", timePoints[i], sep=" "), edge.lwd = 0.1
			, vertex.col = alpha(colors[state], transparency), vertex.border = vertex.border
			, vertex.cex = 0.8, edge.col = edge.col, usearrows = FALSE, mode = "kamadakawai", displayisolates = TRUE) 
	}
}

