# Epidemic model: function to extract and summarize important statistics 
#
# Note: household size categories "2", "3", "4", "5", ">=6"
#
# Author: Nele Goeyvaerts
#
# Last update: 11/06/2014 
#############################################################################

epiSummary <- function(result, display){

# Input parameters:
# 	result: 		output from the epiChainBinModel function (timeSteps, infectedMat, incidenceMat, HH_network, beta2)
# 	display: 		display results (TRUE OR FALSE)
#
# Output parameters:
	##   GLOBAL LEVEL
#	finalSize:		total number of individuals ultimately affected
#	finalFraction:	proportion of individuals ultimately affected
#	incidence:		newly infected individuals over time
#	incidenceChild:	newly infected children over time
#	incidenceFather:	newly infected fathers over time
#	incidenceMother:	newly infected mothers over time
#	extinction:		indicator of extinction i.e. small outbreak (TRUE) or epidemic (FALSE)
#	infPeriod:		duration of infectiousness (NA in case an individual did not get infected)
#	CPIchild:		approximate community probability of infection for children (over the entire epidemic period) 
#	CPIadult:		approximate community probability of infection for adults (over the entire epidemic period) 
	##   HOUSEHOLD LEVEL
#	hhSize:			vector of household size for each household (corresponding to household level statistics)
#	hhFinalSize:		total number of households ultimately affected
#	hhFinalFraction:		proportion of households ultimately affected
#	hhFinalFractionHHsize:	proportion of households ultimately affected per household size category
#	hhIncidence:		newly infected households over time
#	hhPropInfected:		proportion infected in each household
#	hhIndexCase:		role of index case (NA in case no one got infected or 1 single infection)
#	hhLastCase:			role of last case (NA in case no one got infected or 1 single infection)
#	hhDuration:			time from first onset to the last case (NA in case no one got infected or 1 single infection)
#	hhPropInfectedMean: 	mean proportion infected per household size category
#	hhDurationMean:		mean time from first onset to the last case per household size category

	## GLOBAL LEVEL

	# Network considered
	HH_network <- result$HH_network

	# Duration of epidemic
	duration <- result$timeSteps

	# Matrix of infected indicators
	infectedMat <- result$infectedMat
	nCol <- dim(infectedMat)[2]

	# Incidence (newly infected cases) over time
	incidenceMat <- result$incidenceMat
	incidence <- apply(incidenceMat, 2, sum, na.rm = TRUE)
	hh_role <- HH_network %v% "hh_role"
	# Incidence in children (HHs)
	incidenceChild <- apply(incidenceMat[hh_role == "child", ], 2, sum, na.rm = TRUE)
	# Incidence in fathers
	incidenceFather <- apply(incidenceMat[hh_role == "father", ], 2, sum, na.rm = TRUE)
	# Incidence in mothers
	incidenceMother <- apply(incidenceMat[hh_role == "mother", ], 2, sum, na.rm = TRUE)

	# Final size of epidemic
	nrVertices <- network.size(HH_network)
	finalSize <- sum(incidenceMat)
	finalFraction <- finalSize/nrVertices

	# Indicator of extinction
	extinction <- (finalSize < 100) & (duration < 60)		# How to define a small outbreak?
	
	# Dummy variable indicating whether a person has been infected during the epidemic
	infected <- apply(incidenceMat == 1, 1, sum, na.rm = TRUE)

	# Duration of infectiousness (NA in case the individual did not get infected)
	infPeriod <- apply(infectedMat == 1, 1, sum, na.rm = TRUE)
	infPeriod[infected == 0] <- NA
	
	# Community probability of infection (approximation)
	D <- mean(infPeriod, na.rm = TRUE)
	age <- HH_network %v% "age"
	children <- age <= 18
	# Incidence in individuals <= 18 years
	incidenceChildren <- apply(incidenceMat[children, ], 2, sum, na.rm = TRUE)
	# Incidence in individuals > 18 years
	incidenceParents <- apply(incidenceMat[!children, ], 2, sum, na.rm = TRUE)
	nrInfComChild <- sum(incidenceChildren)
	nrInfComAdult <- sum(incidenceParents)
	CPIchild <- 1-((1-beta2[1,1])^(D*nrInfComChild)*(1-beta2[1,2])^(D*nrInfComAdult))
	CPIadult <- 1-((1-beta2[2,2])^(D*nrInfComAdult)*(1-beta2[1,2])^(D*nrInfComChild))


	## HOUSEHOLD LEVEL
	
	# Extract hh_id and hh_size
	hh_id <- HH_network %v% "hh_id"
	hh_size <- HH_network %v% "hh_size"
	hh_id_unique <- unique(hh_id)
	hhSize <- hh_size[!duplicated(hh_id)]

	# Proportion infected in each household
	hhPropInfected <- NULL
	# Index case
	hhIndexCase <- NULL
	# Last case
	hhLastCase <- NULL
	# Time from first onset to the last case
	hhDuration <- NULL
	# Incidence of households over time
	hh_level_incidence <- matrix(0, nrow = length(hh_id_unique), ncol = result$timeSteps)
	for (i in 1:length(hh_id_unique)){
		hh_infected <- NA
		hh_infected <- infected[hh_id == hh_id_unique[i]]
		hhPropInfected[i] <- sum(hh_infected)/length(hh_infected)
		hh_incidence <- NA
		hh_incidence_all <- incidenceMat[hh_id == hh_id_unique[i], ]
		hh_incidence <- apply(hh_incidence_all, 2, sum, na.rm = TRUE)
		hh_infection_times <- which(hh_incidence != 0)
		if (sum(hh_incidence) > 1){ 	# NA in case no one got infected or 1 single infection
			hhComposition <- NA
			hhComposition <- hh_role[hh_id == hh_id_unique[i]]
			hhIndexCase[i] <- hhComposition[hh_incidence_all[, min(hh_infection_times)] == 1][1]	# first element of index case vector
			hhLastCase[i] <- hhComposition[hh_incidence_all[, max(hh_infection_times)] == 1][1]		# first element of last case vector
			hhDuration[i] <- max(hh_infection_times) - min(hh_infection_times)
		}else{
			hhIndexCase[i] <- NA
			hhLastCase[i] <- NA
			hhDuration[i] <- NA
			}
		if (sum(hh_incidence) >= 1){ 	# Corrected on 08/04/2014
			hh_level_incidence[i, hh_infection_times[1]] <- 1
			}
		}
	# Incidence of households over time
	hhIncidence <- apply(hh_level_incidence, 2, sum, na.rm = TRUE)

	# Final size households
	hhFinalSize <- sum(hhIncidence)
	hhFinalFraction <- hhFinalSize/length(hh_id_unique)
	hhAffected <- apply(hh_level_incidence, 1, sum, na.rm = TRUE)

	# Mean proportion infected and mean time from first onset to the last case per HH size
	range <- c("2","3","4","5",">=6")
	hhSize_factor <- cut(hhSize, c(2:6,10), labels = range, right = FALSE)
	hhPropInfectedMean <- NULL
	hhDurationMean <- NULL
	hhFinalFractionHHsize <- NULL
	for (i in 1:length(range)){
		hhPropInfectedMean[i] <- mean(hhPropInfected[hhSize_factor == range[i]])
		hhDurationMean[i] <- mean(hhDuration[hhSize_factor == range[i]], na.rm = TRUE)
		hhFinalFractionHHsize[i] <- mean(hhAffected[hhSize_factor == range[i]], na.rm = TRUE)
		}

	## LIST OF OUTPUT VARIABLES
	output <- list(finalSize = finalSize, finalFraction = finalFraction, incidence = incidence, incidenceChild = incidenceChild
		, incidenceFather = incidenceFather, incidenceMother = incidenceMother, extinction = extinction
		, infPeriod = infPeriod, CPIchild = CPIchild, CPIadult = CPIadult, hhSize = hhSize, hhFinalSize = hhFinalSize, hhFinalFraction = hhFinalFraction
		, hhFinalFractionHHsize = hhFinalFractionHHsize, hhIncidence = hhIncidence, hhPropInfected = hhPropInfected
		, hhIndexCase = hhIndexCase, hhLastCase = hhLastCase, hhDuration = hhDuration, hhPropInfectedMean = hhPropInfectedMean
		, hhDurationMean = hhDurationMean)

	## PRINT OUTPUT VARIABLES
	if (display == TRUE){

	print(output)

	print("Duration epidemic")	
	print(duration)
	
	print("Summary infectious period")
	print(summary(infPeriod))
	
	print("Mean proportion infected per household")
	for (i in 1:length(range)){
		print(paste("Household size ", range[i], sep =""))
		print(hhPropInfectedMean[i])
		}
	
	print("Mean time from first onset to the last case per household")
	for (i in 1:length(range)){
		print(paste("Household size ", range[i], sep =""))
		print(hhDurationMean[i])
		}
	}

	## RETURN OUTPUT
	return(output)
	}












