# Epidemic model: function to extract and summarize important statistics 
#
# Note: household size categories "2", "3", "4", "5", ">=6"
#
# Author: Nele Goeyvaerts
# Based on:
#   Author: Nele Goeyvaerts
#   Last update: 11/06/2014
#
# Last update: 29/07/2016
#############################################################################

epiSummary <- function(result, display){

# Input parameters:
# 	result: 		output from the epiChainBinModel function (timeSteps, infectedMat, incidenceMat, HH_network, beta2)
# 	display: 		display results (TRUE OR FALSE)
#
# Output parameters:
	##   GLOBAL LEVEL
#	finalSize:		total number of individuals ultimately affected
#	finalFraction:	proportion of individuals ultimately affected
#	incidence:		newly infected individuals over time
#	incidenceChild:	newly infected children over time
#	incidenceFather:	newly infected fathers over time
#	incidenceMother:	newly infected mothers over time
#	extinction:		indicator of extinction i.e. small outbreak (TRUE) or epidemic (FALSE)
#	infPeriod:		duration of infectiousness (NA in case an individual did not get infected)
#	CPIchild:		approximate community probability of infection for children (over the entire epidemic period) 
#	CPIadult:		approximate community probability of infection for adults (over the entire epidemic period) 
# hhfoiMean: FOI from household averaged over all susceptible individuals
# comfoiMean_child: FOI from children in community averaged over all susceptible individuals
# comfoiMean_adult: FOI from adults in community averaged over all susceptible individuals
  
	##   HOUSEHOLD LEVEL
#	hhSize:			vector of household size for each household (corresponding to household level statistics)
#	hhFinalSize:		total number of households ultimately affected
#	hhFinalFraction:		proportion of households ultimately affected
#	hhFinalFractionHHsize:	proportion of households ultimately affected per household size category
#	hhIncidence:		newly infected households over time
# hhIncidence_hhsize:		newly infected households over time per household size category
#	hhPropInfected:		proportion infected in each household
#	hhIndexCase:		role of index case (NA in case no one got infected or 1 single infection)
#	hhLastCase:			role of last case (NA in case no one got infected or 1 single infection)
#	hhDuration:			time from first onset to the last case (NA in case no one got infected or 1 single infection)
#	hhPropInfectedMean: 	mean proportion infected per household size category
# hhSecInfectedMean: mean proportion infected in affected households per household size category
# hhInfectedTable: distribution number of infections per household size category
#	hhDurationMean:		mean time from first onset to the last case per household size category
# hhPropInfectedAll: 	proportion infected per household 
# hhfoi_hhsize: FOI from household averaged over all susceptible individuals per household size category
# comfoi_child_hhsize: FOI from children in community averaged over all susceptible individuals per household size category
# comfoi_adult_hhsize: FOI from adults in community averaged over all susceptible individuals per household size category
  

  ## GLOBAL LEVEL

	# Network considered
	HH_network <- result$HH_network

	# Duration of epidemic
	duration <- result$timeSteps

	# Matrix of infected indicators
	infectedMat <- result$infectedMat
	nCol <- dim(infectedMat)[2]

	# Incidence (newly infected cases) over time
	incidenceMat <- result$incidenceMat
	incidence <- apply(incidenceMat, 2, sum, na.rm = TRUE)
	hh_role <- HH_network %v% "hh_role"
	# Incidence in children (HHs)
	incidenceChild <- apply(incidenceMat[hh_role == "child", ], 2, sum, na.rm = TRUE)
	# Incidence in fathers
	incidenceFather <- apply(incidenceMat[hh_role == "father", ], 2, sum, na.rm = TRUE)
	# Incidence in mothers
	incidenceMother <- apply(incidenceMat[hh_role == "mother", ], 2, sum, na.rm = TRUE)
  
  # Number of cases over time
  cases <- apply(infectedMat, 2, sum, na.rm=T)
  
  # Force of infection
  hhfoiMat <- result$hhfoiMat
  comfoiMat_child <- result$comfoiMat_child
	comfoiMat_adult <- result$comfoiMat_adult
  # Mean foi
  hhfoiMean <- apply(hhfoiMat,2,mean,na.rm=T)
  comfoiMean_child <- apply(comfoiMat_child,2,mean,na.rm=T)
  comfoiMean_adult <- apply(comfoiMat_adult,2,mean,na.rm=T)

	# Final size of epidemic
	nrVertices <- network.size(HH_network)
	finalSize <- sum(incidenceMat)
	finalFraction <- finalSize/nrVertices

	# Indicator of extinction
	extinction <- (finalSize < 100) & (duration < 60)		# How to define a small outbreak?
	
	# Dummy variable indicating whether a person has been infected during the epidemic
	infected <- apply(incidenceMat == 1, 1, sum, na.rm = TRUE)

	# Duration of infectiousness (NA in case the individual did not get infected)
	infPeriod <- apply(infectedMat == 1, 1, sum, na.rm = TRUE)
	infPeriod[infected == 0] <- NA
	
	# Community probability of infection (approximation)
	D <- mean(infPeriod, na.rm = TRUE)
	age <- HH_network %v% "age"
	children <- age <= 18
	# Incidence in individuals <= 18 years
	incidenceChildren <- apply(incidenceMat[children, ], 2, sum, na.rm = TRUE)
	# Incidence in individuals > 18 years
	incidenceParents <- apply(incidenceMat[!children, ], 2, sum, na.rm = TRUE)
	nrInfComChild <- sum(incidenceChildren)
	nrInfComAdult <- sum(incidenceParents)
	CPIchild <- 1-((1-beta2[1,1])^(D*nrInfComChild)*(1-beta2[1,2])^(D*nrInfComAdult))
	CPIadult <- 1-((1-beta2[2,2])^(D*nrInfComAdult)*(1-beta2[1,2])^(D*nrInfComChild))


	## HOUSEHOLD LEVEL
	
	# Extract hh_id and hh_size
	hh_id <- HH_network %v% "hh_id"
	hh_size <- HH_network %v% "hh_size"
	hh_id_unique <- unique(hh_id)
	hhSize <- hh_size[!duplicated(hh_id)]

	# Number infected in each household
	hhInfected <- NULL
  # Proportion infected in each household
	hhPropInfected <- NULL
	# Index case
	hhIndexCase <- rep(NA,length(hh_id_unique))
	hhIndexCaseAll <- rep(NA,length(hh_id_unique))
	# Last case
	hhLastCase <- rep(NA,length(hh_id_unique))
	# Time from first onset to the last case
	hhDuration <- rep(NA,length(hh_id_unique))
	# Incidence of households over time
	hh_level_incidence <- matrix(0, nrow = length(hh_id_unique), ncol = result$timeSteps)
  ## Mean foi in households over time
	hh_hhfoi <- matrix(NA, nrow = length(hh_id_unique), ncol = result$timeSteps-1)
	hh_comfoi_child <- matrix(NA, nrow = length(hh_id_unique), ncol = result$timeSteps-1)
	hh_comfoi_adult <- matrix(NA, nrow = length(hh_id_unique), ncol = result$timeSteps-1)
	for (i in 1:length(hh_id_unique)){
		hh_infected <- NA
		hh_infected <- infected[hh_id == hh_id_unique[i]]
		hhInfected[i] <- sum(hh_infected)
		hhPropInfected[i] <- hhInfected[i]/length(hh_infected)
		hh_incidence <- NA
		hh_incidence_all <- incidenceMat[hh_id == hh_id_unique[i], ]
		hh_incidence <- apply(hh_incidence_all, 2, sum, na.rm = TRUE)
		hh_infection_times <- which(hh_incidence != 0)
    hh_hhfoi[i,] <- apply(as.matrix(hhfoiMat[hh_id == hh_id_unique[i], ]),2,mean,na.rm=T)
    hh_comfoi_child[i,] <- apply(as.matrix(comfoiMat_child[hh_id == hh_id_unique[i], ]),2,mean,na.rm=T)
    hh_comfoi_adult[i,] <- apply(as.matrix(comfoiMat_adult[hh_id == hh_id_unique[i], ]),2,mean,na.rm=T)
		if (sum(hh_incidence) > 1){ 	# NA in case no one got infected or 1 single infection
			hhComposition <- NA
			hhComposition <- hh_role[hh_id == hh_id_unique[i]]
			hhIndexCase[i] <- hhComposition[hh_incidence_all[, min(hh_infection_times)] == 1][1]	# first element of index case vector
			hhLastCase[i] <- hhComposition[hh_incidence_all[, max(hh_infection_times)] == 1][1]		# first element of last case vector
			hhDuration[i] <- max(hh_infection_times) - min(hh_infection_times)
		}
		if (sum(hh_incidence) >= 1){ 	# Corrected on 08/04/2014
			hh_level_incidence[i, hh_infection_times[1]] <- 1
			hhComposition <- NA
			hhComposition <- hh_role[hh_id == hh_id_unique[i]]
			hhIndexCaseAll[i] <- hhComposition[hh_incidence_all[, min(hh_infection_times)] == 1][1]
			}
		}
	# Incidence of households over time
	hhIncidence <- apply(hh_level_incidence, 2, sum, na.rm = TRUE)

	# Final size households
	hhFinalSize <- sum(hhIncidence)
	hhFinalFraction <- hhFinalSize/length(hh_id_unique)
	hhAffected <- apply(hh_level_incidence, 1, sum, na.rm = TRUE)

	# Distribution number infected, mean proportion infected and mean time from first onset to the last case per HH size
	range <- c("2","3","4","5",">=6")
	hhSize_factor <- cut(hhSize, c(2:6,10), labels = range, right = FALSE)
	hhInfected_factor <- cut(hhInfected, c(0:6,10), labels = c("0","1","2","3","4","5",">=6"), right = FALSE)
	hhInfectedTable <- matrix(NA, nrow = length(range), ncol = length(range)+2)
	hhPropInfectedMean <- NULL
  hhSecInfectedMean <- NULL
	hhDurationMean <- NULL
	hhFinalFractionHHsize <- NULL
	hhIncidence_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_level_incidence))
	hhfoi_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_hhfoi))
	comfoi_child_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_comfoi_child))
	comfoi_adult_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_comfoi_adult))
	for (i in 1:length(range)){
	  hhPropInfected_hhsize <- hhPropInfected[hhSize_factor == range[i]]
	  hhPropInfectedMean[i] <- mean(hhPropInfected_hhsize)
    hhSecInfectedMean[i] <- mean(hhPropInfected_hhsize[hhPropInfected_hhsize!=0])
		hhInfectedTable[i,] <- table(hhInfected_factor[hhSize_factor == range[i]])
		hhDurationMean[i] <- mean(hhDuration[hhSize_factor == range[i]], na.rm = TRUE)
		hhFinalFractionHHsize[i] <- mean(hhAffected[hhSize_factor == range[i]], na.rm = TRUE)
    hhIncidence_hhsize[i,] <- apply(hh_level_incidence[hhSize_factor == range[i],], 2, sum, na.rm = TRUE)
    hhfoi_hhsize[i,] <- apply(as.matrix(hh_hhfoi[hhSize_factor == range[i],]), 2, mean, na.rm = TRUE)
		comfoi_child_hhsize[i,] <- apply(as.matrix(hh_comfoi_child[hhSize_factor == range[i],]), 2, mean, na.rm = TRUE)
		comfoi_adult_hhsize[i,] <- apply(as.matrix(hh_comfoi_adult[hhSize_factor == range[i],]), 2, mean, na.rm = TRUE)
	}

	## LIST OF OUTPUT VARIABLES
	output <- list(finalSize = finalSize, finalFraction = finalFraction, extinction = extinction
	  , incidence = incidence, incidenceChild = incidenceChild, incidenceFather = incidenceFather, incidenceMother = incidenceMother
	  , cases = cases, infPeriod = infPeriod, CPIchild = CPIchild, CPIadult = CPIadult
     , hhSize = hhSize, hhFinalSize = hhFinalSize, hhFinalFraction = hhFinalFraction, hhPropInfected = hhPropInfected, hhIncidence = hhIncidence
	   , hhIndexCase = hhIndexCase, hhIndexCaseAll = hhIndexCaseAll, hhLastCase = hhLastCase, hhDuration = hhDuration
	   , hhfoiMean = hhfoiMean, comfoiMean_child = comfoiMean_child, comfoiMean_adult = comfoiMean_adult
      , hhPropInfectedMean = hhPropInfectedMean, hhSecInfectedMean = hhSecInfectedMean, hhInfectedTable = hhInfectedTable
		  , hhDurationMean = hhDurationMean, hhIncidence_hhsize = hhIncidence_hhsize, hhFinalFractionHHsize = hhFinalFractionHHsize   
  	  , hhfoi_hhsize = hhfoi_hhsize, comfoi_child_hhsize = comfoi_child_hhsize, comfoi_adult_hhsize = comfoi_adult_hhsize)

	## PRINT OUTPUT VARIABLES
	if (display == TRUE){

	print(output)

	print("Duration epidemic")	
	print(duration)
	
	print("Summary infectious period")
	print(summary(infPeriod))
	
	print("Mean proportion infected per household")
	for (i in 1:length(range)){
		print(paste("Household size ", range[i], sep =""))
		print(hhPropInfectedMean[i])
		}
	
	print("Mean time from first onset to the last case per household")
	for (i in 1:length(range)){
		print(paste("Household size ", range[i], sep =""))
		print(hhDurationMean[i])
		}
	}

	## RETURN OUTPUT
	return(output)
	}


## age-dependent hh transmission
epiSummary11 <- function(result, display){
  
  # Input parameters:
  #   result: 		output from the epiChainBinModel function (timeSteps, infectedMat, incidenceMat, HH_network, beta2)
  # 	display: 		display results (TRUE OR FALSE)
  #
  # Output parameters:
  ##   GLOBAL LEVEL
  #	finalSize:		total number of individuals ultimately affected
  #	finalFraction:	proportion of individuals ultimately affected
  #	incidence:		newly infected individuals over time
  #	incidenceChild:	newly infected children over time
  #	incidenceFather:	newly infected fathers over time
  #	incidenceMother:	newly infected mothers over time
  #	extinction:		indicator of extinction i.e. small outbreak (TRUE) or epidemic (FALSE)
  #	infPeriod:		duration of infectiousness (NA in case an individual did not get infected)
  #	CPIchild:		approximate community probability of infection for children (over the entire epidemic period) 
  #	CPIadult:		approximate community probability of infection for adults (over the entire epidemic period) 
  # hhfoiMean_child: FOI from household averaged over all susceptible individuals
  # hhfoiMean_adult: FOI from household averaged over all susceptible individuals
  # comfoiMean_child: FOI from children in community averaged over all susceptible individuals
  # comfoiMean_adult: FOI from adults in community averaged over all susceptible individuals
  
  ##   HOUSEHOLD LEVEL
  #	hhSize:			vector of household size for each household (corresponding to household level statistics)
  #	hhFinalSize:		total number of households ultimately affected
  #	hhFinalFraction:		proportion of households ultimately affected
  #	hhFinalFractionHHsize:	proportion of households ultimately affected per household size category
  #	hhIncidence:		newly infected households over time
  # hhIncidence_hhsize:		newly infected households over time per household size category
  #	hhPropInfected:		proportion infected in each household
  #	hhIndexCase:		role of index case (NA in case no one got infected or 1 single infection)
  #	hhLastCase:			role of last case (NA in case no one got infected or 1 single infection)
  #	hhDuration:			time from first onset to the last case (NA in case no one got infected or 1 single infection)
  #	hhPropInfectedMean: 	mean proportion infected per household size category
  # hhSecInfectedMean: mean proportion infected in affected households per household size category
  # hhInfectedTable: distribution number of infections per household size category
  #	hhDurationMean:		mean time from first onset to the last case per household size category
  # hhPropInfectedAll: 	proportion infected per household 
  # hhfoi_child_hhsize: FOI from household averaged over all susceptible individuals per household size category
  # hhfoi_adult_hhsize: FOI from household averaged over all susceptible individuals per household size category
  # comfoi_child_hhsize: FOI from children in community averaged over all susceptible individuals per household size category
  # comfoi_adult_hhsize: FOI from adults in community averaged over all susceptible individuals per household size category
  
  
  ## GLOBAL LEVEL
  
  # Network considered
  HH_network <- result$HH_network
  
  # Duration of epidemic
  duration <- result$timeSteps
  
  # Matrix of infected indicators
  infectedMat <- result$infectedMat
  nCol <- dim(infectedMat)[2]
  
  # Incidence (newly infected cases) over time
  incidenceMat <- result$incidenceMat
  incidence <- apply(incidenceMat, 2, sum, na.rm = TRUE)
  hh_role <- HH_network %v% "hh_role"
  # Incidence in children (HHs)
  incidenceChild <- apply(incidenceMat[hh_role == "child", ], 2, sum, na.rm = TRUE)
  # Incidence in fathers
  incidenceFather <- apply(incidenceMat[hh_role == "father", ], 2, sum, na.rm = TRUE)
  # Incidence in mothers
  incidenceMother <- apply(incidenceMat[hh_role == "mother", ], 2, sum, na.rm = TRUE)
  
  # Force of infection
  hhfoiMat_child <- result$hhfoiMat_child
  hhfoiMat_adult <- result$hhfoiMat_adult
  comfoiMat_child <- result$comfoiMat_child
  comfoiMat_adult <- result$comfoiMat_adult
  # Mean foi
  hhfoiMean_child <- apply(hhfoiMat_child,2,mean,na.rm=T)
  hhfoiMean_adult <- apply(hhfoiMat_adult,2,mean,na.rm=T)
  comfoiMean_child <- apply(comfoiMat_child,2,mean,na.rm=T)
  comfoiMean_adult <- apply(comfoiMat_adult,2,mean,na.rm=T)
  
  # Final size of epidemic
  nrVertices <- network.size(HH_network)
  finalSize <- sum(incidenceMat)
  finalFraction <- finalSize/nrVertices
  
  # Indicator of extinction
  extinction <- (finalSize < 100) & (duration < 60)		# How to define a small outbreak?
  
  # Dummy variable indicating whether a person has been infected during the epidemic
  infected <- apply(incidenceMat == 1, 1, sum, na.rm = TRUE)
  
  # Duration of infectiousness (NA in case the individual did not get infected)
  infPeriod <- apply(infectedMat == 1, 1, sum, na.rm = TRUE)
  infPeriod[infected == 0] <- NA
  
  # Community probability of infection (approximation)
  D <- mean(infPeriod, na.rm = TRUE)
  age <- HH_network %v% "age"
  children <- age <= 18
  # Incidence in individuals <= 18 years
  incidenceChildren <- apply(incidenceMat[children, ], 2, sum, na.rm = TRUE)
  # Incidence in individuals > 18 years
  incidenceParents <- apply(incidenceMat[!children, ], 2, sum, na.rm = TRUE)
  nrInfComChild <- sum(incidenceChildren)
  nrInfComAdult <- sum(incidenceParents)
  CPIchild <- 1-((1-beta2[1,1])^(D*nrInfComChild)*(1-beta2[1,2])^(D*nrInfComAdult))
  CPIadult <- 1-((1-beta2[2,2])^(D*nrInfComAdult)*(1-beta2[1,2])^(D*nrInfComChild))
  
  
  ## HOUSEHOLD LEVEL
  
  # Extract hh_id and hh_size
  hh_id <- HH_network %v% "hh_id"
  hh_size <- HH_network %v% "hh_size"
  hh_id_unique <- unique(hh_id)
  hhSize <- hh_size[!duplicated(hh_id)]
  
  # Number infected in each household
  hhInfected <- NULL
  # Proportion infected in each household
  hhPropInfected <- NULL
  # Index case
  hhIndexCase <- rep(NA,length(hh_id_unique))
  hhIndexCaseAll <- rep(NA,length(hh_id_unique))
  # Last case
  hhLastCase <- rep(NA,length(hh_id_unique))
  # Time from first onset to the last case
  hhDuration <- rep(NA,length(hh_id_unique))
  # Incidence of households over time
  hh_level_incidence <- matrix(0, nrow = length(hh_id_unique), ncol = result$timeSteps)
  ## Mean foi in households over time
  hh_hhfoi_child <- matrix(NA, nrow = length(hh_id_unique), ncol = result$timeSteps-1)
  hh_hhfoi_adult <- matrix(NA, nrow = length(hh_id_unique), ncol = result$timeSteps-1)
  hh_comfoi_child <- matrix(NA, nrow = length(hh_id_unique), ncol = result$timeSteps-1)
  hh_comfoi_adult <- matrix(NA, nrow = length(hh_id_unique), ncol = result$timeSteps-1)
  for (i in 1:length(hh_id_unique)){
    hh_infected <- NA
    hh_infected <- infected[hh_id == hh_id_unique[i]]
    hhInfected[i] <- sum(hh_infected)
    hhPropInfected[i] <- hhInfected[i]/length(hh_infected)
    hh_incidence <- NA
    hh_incidence_all <- incidenceMat[hh_id == hh_id_unique[i], ]
    hh_incidence <- apply(hh_incidence_all, 2, sum, na.rm = TRUE)
    hh_infection_times <- which(hh_incidence != 0)
    hh_hhfoi_child[i,] <- apply(as.matrix(hhfoiMat_child[hh_id == hh_id_unique[i], ]),2,mean,na.rm=T)
    hh_hhfoi_adult[i,] <- apply(as.matrix(hhfoiMat_adult[hh_id == hh_id_unique[i], ]),2,mean,na.rm=T)
    hh_comfoi_child[i,] <- apply(as.matrix(comfoiMat_child[hh_id == hh_id_unique[i], ]),2,mean,na.rm=T)
    hh_comfoi_adult[i,] <- apply(as.matrix(comfoiMat_adult[hh_id == hh_id_unique[i], ]),2,mean,na.rm=T)
    if (sum(hh_incidence) > 1){ 	# NA in case no one got infected or 1 single infection
      hhComposition <- NA
      hhComposition <- hh_role[hh_id == hh_id_unique[i]]
      hhIndexCase[i] <- hhComposition[hh_incidence_all[, min(hh_infection_times)] == 1][1]	# first element of index case vector
      hhLastCase[i] <- hhComposition[hh_incidence_all[, max(hh_infection_times)] == 1][1]		# first element of last case vector
      hhDuration[i] <- max(hh_infection_times) - min(hh_infection_times)
    }
    if (sum(hh_incidence) >= 1){ 	# Corrected on 08/04/2014
      hh_level_incidence[i, hh_infection_times[1]] <- 1
      hhComposition <- NA
      hhComposition <- hh_role[hh_id == hh_id_unique[i]]
      hhIndexCaseAll[i] <- hhComposition[hh_incidence_all[, min(hh_infection_times)] == 1][1]
    }
  }
  # Incidence of households over time
  hhIncidence <- apply(hh_level_incidence, 2, sum, na.rm = TRUE)
  
  # Final size households
  hhFinalSize <- sum(hhIncidence)
  hhFinalFraction <- hhFinalSize/length(hh_id_unique)
  hhAffected <- apply(hh_level_incidence, 1, sum, na.rm = TRUE)
  
  # Distribution number infected, mean proportion infected and mean time from first onset to the last case per HH size
  range <- c("2","3","4","5",">=6")
  hhSize_factor <- cut(hhSize, c(2:6,10), labels = range, right = FALSE)
  hhInfected_factor <- cut(hhInfected, c(0:6,10), labels = c("0","1","2","3","4","5",">=6"), right = FALSE)
  hhInfectedTable <- matrix(NA, nrow = length(range), ncol = length(range)+2)
  hhPropInfectedMean <- NULL
  hhSecInfectedMean <- NULL
  hhDurationMean <- NULL
  hhFinalFractionHHsize <- NULL
  hhIncidence_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_level_incidence))
  hhfoi_child_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_hhfoi_child))
  hhfoi_adult_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_hhfoi_adult))
  comfoi_child_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_comfoi_child))
  comfoi_adult_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_comfoi_adult))
  for (i in 1:length(range)){
    hhPropInfected_hhsize <- hhPropInfected[hhSize_factor == range[i]]
    hhPropInfectedMean[i] <- mean(hhPropInfected_hhsize)
    hhSecInfectedMean[i] <- mean(hhPropInfected_hhsize[hhPropInfected_hhsize!=0])
    hhInfectedTable[i,] <- table(hhInfected_factor[hhSize_factor == range[i]])
    hhDurationMean[i] <- mean(hhDuration[hhSize_factor == range[i]], na.rm = TRUE)
    hhFinalFractionHHsize[i] <- mean(hhAffected[hhSize_factor == range[i]], na.rm = TRUE)
    hhIncidence_hhsize[i,] <- apply(hh_level_incidence[hhSize_factor == range[i],], 2, sum, na.rm = TRUE)
    hhfoi_child_hhsize[i,] <- apply(as.matrix(hh_hhfoi_child[hhSize_factor == range[i],]), 2, mean, na.rm = TRUE)
    hhfoi_adult_hhsize[i,] <- apply(as.matrix(hh_hhfoi_adult[hhSize_factor == range[i],]), 2, mean, na.rm = TRUE)
    comfoi_child_hhsize[i,] <- apply(as.matrix(hh_comfoi_child[hhSize_factor == range[i],]), 2, mean, na.rm = TRUE)
    comfoi_adult_hhsize[i,] <- apply(as.matrix(hh_comfoi_adult[hhSize_factor == range[i],]), 2, mean, na.rm = TRUE)
  }
  
  ## LIST OF OUTPUT VARIABLES
  output <- list(finalSize = finalSize, finalFraction = finalFraction, extinction = extinction
                 , incidence = incidence, incidenceChild = incidenceChild, incidenceFather = incidenceFather, incidenceMother = incidenceMother
                 , infPeriod = infPeriod, CPIchild = CPIchild, CPIadult = CPIadult
                 , hhSize = hhSize, hhFinalSize = hhFinalSize, hhFinalFraction = hhFinalFraction, hhPropInfected = hhPropInfected, hhIncidence = hhIncidence
                 , hhIndexCase = hhIndexCase, hhIndexCaseAll = hhIndexCaseAll, hhLastCase = hhLastCase, hhDuration = hhDuration
                 , hhfoiMean_child = hhfoiMean_child, hhfoiMean_adult = hhfoiMean_adult, comfoiMean_child = comfoiMean_child, comfoiMean_adult = comfoiMean_adult
                 , hhPropInfectedMean = hhPropInfectedMean, hhSecInfectedMean = hhSecInfectedMean, hhInfectedTable = hhInfectedTable
                 , hhDurationMean = hhDurationMean, hhIncidence_hhsize = hhIncidence_hhsize, hhFinalFractionHHsize = hhFinalFractionHHsize   
                 , hhfoi_child_hhsize = hhfoi_child_hhsize, hhfoi_adult_hhsize = hhfoi_adult_hhsize, comfoi_child_hhsize = comfoi_child_hhsize, comfoi_adult_hhsize = comfoi_adult_hhsize)
  
  ## PRINT OUTPUT VARIABLES
  if (display == TRUE){
    
    print(output)
    
    print("Duration epidemic")	
    print(duration)
    
    print("Summary infectious period")
    print(summary(infPeriod))
    
    print("Mean proportion infected per household")
    for (i in 1:length(range)){
      print(paste("Household size ", range[i], sep =""))
      print(hhPropInfectedMean[i])
    }
    
    print("Mean time from first onset to the last case per household")
    for (i in 1:length(range)){
      print(paste("Household size ", range[i], sep =""))
      print(hhDurationMean[i])
    }
  }
  
  ## RETURN OUTPUT
  return(output)
}


## Without FOI
epiSummary2 <- function(result, display){
  
  # Input parameters:
  #   result: 		output from the epiChainBinModel function (timeSteps, infectedMat, incidenceMat, HH_network, beta2)
  # 	display: 		display results (TRUE OR FALSE)
  #
  # Output parameters:
  ##   GLOBAL LEVEL
  #	finalSize:		total number of individuals ultimately affected
  #	finalFraction:	proportion of individuals ultimately affected
  #	incidence:		newly infected individuals over time
  #	incidenceChild:	newly infected children over time
  #	incidenceFather:	newly infected fathers over time
  #	incidenceMother:	newly infected mothers over time
  #	extinction:		indicator of extinction i.e. small outbreak (TRUE) or epidemic (FALSE)
  #	infPeriod:		duration of infectiousness (NA in case an individual did not get infected)
  #	CPIchild:		approximate community probability of infection for children (over the entire epidemic period) 
  #	CPIadult:		approximate community probability of infection for adults (over the entire epidemic period) 
  
  ##   HOUSEHOLD LEVEL
  #	hhSize:			vector of household size for each household (corresponding to household level statistics)
  #	hhFinalSize:		total number of households ultimately affected
  #	hhFinalFraction:		proportion of households ultimately affected
  #	hhFinalFractionHHsize:	proportion of households ultimately affected per household size category
  #	hhIncidence:		newly infected households over time
  # hhIncidence_hhsize:		newly infected households over time per household size category
  #	hhPropInfected:		proportion infected in each household
  #	hhIndexCase:		role of index case (NA in case no one got infected or 1 single infection)
  #	hhLastCase:			role of last case (NA in case no one got infected or 1 single infection)
  #	hhDuration:			time from first onset to the last case (NA in case no one got infected or 1 single infection)
  #	hhPropInfectedMean: 	mean proportion infected per household size category
  # hhSecInfectedMean: mean proportion infected in affected households per household size category
  # hhInfectedTable: distribution number of infections per household size category
  #	hhDurationMean:		mean time from first onset to the last case per household size category
  # hhPropInfectedAll: 	proportion infected per household 
  
  
  ## GLOBAL LEVEL
  
  # Network considered
  HH_network <- result$HH_network
  
  # Duration of epidemic
  duration <- result$timeSteps
  
  # Matrix of infected indicators
  infectedMat <- result$infectedMat
  nCol <- dim(infectedMat)[2]
  
  # Incidence (newly infected cases) over time
  incidenceMat <- result$incidenceMat
  incidence <- apply(incidenceMat, 2, sum, na.rm = TRUE)
  hh_role <- HH_network %v% "hh_role"
  # Incidence in children (HHs)
  incidenceChild <- apply(incidenceMat[hh_role == "child", ], 2, sum, na.rm = TRUE)
  # Incidence in fathers
  incidenceFather <- apply(incidenceMat[hh_role == "father", ], 2, sum, na.rm = TRUE)
  # Incidence in mothers
  incidenceMother <- apply(incidenceMat[hh_role == "mother", ], 2, sum, na.rm = TRUE)
  
  # Final size of epidemic
  nrVertices <- network.size(HH_network)
  finalSize <- sum(incidenceMat)
  finalFraction <- finalSize/nrVertices
  
  # Indicator of extinction
  extinction <- (finalSize < 100) & (duration < 60)		# How to define a small outbreak?
  
  # Dummy variable indicating whether a person has been infected during the epidemic
  infected <- apply(incidenceMat == 1, 1, sum, na.rm = TRUE)
  
  # Duration of infectiousness (NA in case the individual did not get infected)
  infPeriod <- apply(infectedMat == 1, 1, sum, na.rm = TRUE)
  infPeriod[infected == 0] <- NA
  
  # Community probability of infection (approximation)
  D <- mean(infPeriod, na.rm = TRUE)
  age <- HH_network %v% "age"
  children <- age <= 18
  # Incidence in individuals <= 18 years
  incidenceChildren <- apply(incidenceMat[children, ], 2, sum, na.rm = TRUE)
  # Incidence in individuals > 18 years
  incidenceParents <- apply(incidenceMat[!children, ], 2, sum, na.rm = TRUE)
  nrInfComChild <- sum(incidenceChildren)
  nrInfComAdult <- sum(incidenceParents)
  CPIchild <- 1-((1-beta2[1,1])^(D*nrInfComChild)*(1-beta2[1,2])^(D*nrInfComAdult))
  CPIadult <- 1-((1-beta2[2,2])^(D*nrInfComAdult)*(1-beta2[1,2])^(D*nrInfComChild))
  
  
  ## HOUSEHOLD LEVEL
  
  # Extract hh_id and hh_size
  hh_id <- HH_network %v% "hh_id"
  hh_size <- HH_network %v% "hh_size"
  hh_id_unique <- unique(hh_id)
  hhSize <- hh_size[!duplicated(hh_id)]
  
  # Number infected in each household
  hhInfected <- NULL
  # Proportion infected in each household
  hhPropInfected <- NULL
  # Index case
  hhIndexCase <- rep(NA,length(hh_id_unique))
  hhIndexCaseAll <- rep(NA,length(hh_id_unique))
  # Last case
  hhLastCase <- rep(NA,length(hh_id_unique))
  # Time from first onset to the last case
  hhDuration <- rep(NA,length(hh_id_unique))
  # Incidence of households over time
  hh_level_incidence <- matrix(0, nrow = length(hh_id_unique), ncol = result$timeSteps)
  for (i in 1:length(hh_id_unique)){
    hh_infected <- NA
    hh_infected <- infected[hh_id == hh_id_unique[i]]
    hhInfected[i] <- sum(hh_infected)
    hhPropInfected[i] <- hhInfected[i]/length(hh_infected)
    hh_incidence <- NA
    hh_incidence_all <- incidenceMat[hh_id == hh_id_unique[i], ]
    hh_incidence <- apply(hh_incidence_all, 2, sum, na.rm = TRUE)
    hh_infection_times <- which(hh_incidence != 0)
    if (sum(hh_incidence) > 1){ 	# NA in case no one got infected or 1 single infection
      hhComposition <- NA
      hhComposition <- hh_role[hh_id == hh_id_unique[i]]
      hhIndexCase[i] <- hhComposition[hh_incidence_all[, min(hh_infection_times)] == 1][1]	# first element of index case vector
      hhLastCase[i] <- hhComposition[hh_incidence_all[, max(hh_infection_times)] == 1][1]		# first element of last case vector
      hhDuration[i] <- max(hh_infection_times) - min(hh_infection_times)
    }
    if (sum(hh_incidence) >= 1){ 	# Corrected on 08/04/2014
      hh_level_incidence[i, hh_infection_times[1]] <- 1
      hhComposition <- NA
      hhComposition <- hh_role[hh_id == hh_id_unique[i]]
      hhIndexCaseAll[i] <- hhComposition[hh_incidence_all[, min(hh_infection_times)] == 1][1]
    }
  }
  # Incidence of households over time
  hhIncidence <- apply(hh_level_incidence, 2, sum, na.rm = TRUE)
  
  # Final size households
  hhFinalSize <- sum(hhIncidence)
  hhFinalFraction <- hhFinalSize/length(hh_id_unique)
  hhAffected <- apply(hh_level_incidence, 1, sum, na.rm = TRUE)
  
  # Distribution number infected, mean proportion infected and mean time from first onset to the last case per HH size
  range <- c("2","3","4","5",">=6")
  hhSize_factor <- cut(hhSize, c(2:6,10), labels = range, right = FALSE)
  hhInfected_factor <- cut(hhInfected, c(0:6,10), labels = c("0","1","2","3","4","5",">=6"), right = FALSE)
  hhInfectedTable <- matrix(NA, nrow = length(range), ncol = length(range)+2)
  hhPropInfectedMean <- NULL
  hhSecInfectedMean <- NULL
  hhDurationMean <- NULL
  hhFinalFractionHHsize <- NULL
  hhIncidence_hhsize <- matrix(NA, nrow = length(range), ncol = ncol(hh_level_incidence))
  for (i in 1:length(range)){
    hhPropInfected_hhsize <- hhPropInfected[hhSize_factor == range[i]]
    hhPropInfectedMean[i] <- mean(hhPropInfected_hhsize)
    hhSecInfectedMean[i] <- mean(hhPropInfected_hhsize[hhPropInfected_hhsize!=0])
    hhInfectedTable[i,] <- table(hhInfected_factor[hhSize_factor == range[i]])
    hhDurationMean[i] <- mean(hhDuration[hhSize_factor == range[i]], na.rm = TRUE)
    hhFinalFractionHHsize[i] <- mean(hhAffected[hhSize_factor == range[i]], na.rm = TRUE)
    hhIncidence_hhsize[i,] <- apply(hh_level_incidence[hhSize_factor == range[i],], 2, sum, na.rm = TRUE)
  }
  
  ## LIST OF OUTPUT VARIABLES
  output <- list(finalSize = finalSize, finalFraction = finalFraction, extinction = extinction
                 , incidence = incidence, incidenceChild = incidenceChild, incidenceFather = incidenceFather, incidenceMother = incidenceMother
                 , infPeriod = infPeriod, CPIchild = CPIchild, CPIadult = CPIadult
                 , hhSize = hhSize, hhFinalSize = hhFinalSize, hhFinalFraction = hhFinalFraction, hhPropInfected = hhPropInfected, hhIncidence = hhIncidence
                 , hhIndexCase = hhIndexCase, hhIndexCaseAll = hhIndexCaseAll, hhLastCase = hhLastCase, hhDuration = hhDuration
                 , hhPropInfectedMean = hhPropInfectedMean, hhSecInfectedMean = hhSecInfectedMean, hhInfectedTable = hhInfectedTable
                 , hhDurationMean = hhDurationMean, hhIncidence_hhsize = hhIncidence_hhsize, hhFinalFractionHHsize = hhFinalFractionHHsize   
                 )
  
  ## PRINT OUTPUT VARIABLES
  if (display == TRUE){
    
    print(output)
    
    print("Duration epidemic")	
    print(duration)
    
    print("Summary infectious period")
    print(summary(infPeriod))
    
    print("Mean proportion infected per household")
    for (i in 1:length(range)){
      print(paste("Household size ", range[i], sep =""))
      print(hhPropInfectedMean[i])
    }
    
    print("Mean time from first onset to the last case per household")
    for (i in 1:length(range)){
      print(paste("Household size ", range[i], sep =""))
      print(hhDurationMean[i])
    }
  }
  
  ## RETURN OUTPUT
  return(output)
}

## Independent households: no community transmission
epiSummary_ind <- function(result, display){
  
  ## Input parameters:
  #   result:   	output from the epiChainBinModel function (timeSteps, statusMat, incidenceMat, HH_network)
  # 	display: 		display results (TRUE OR FALSE)
  ## Output parameters:
  #	  GLOBAL LEVEL
  #	    finalSize:		total number of individuals ultimately affected
  #	    finalFraction:	proportion of individuals ultimately affected
  #	    latentPeriod:		duration of latent period (NA in case an individual did not get infected)
  #	  HOUSEHOLD LEVEL
  #	    hhSize:		vector of household size for each household (corresponding to HH level statistics)
  #     hhFinalSize:	total number of households with more than 1 infection
  #	    hhFinalFraction:	proportion of households with more than 1 infection
  #	    hhPropInfected:	proportion infected in each household
  #	    hhDuration:		time from first onset to the last case (0 in case only 1 infection)
  #     hhObservation: duration of observation period
  #     hhIncidence: number of new cases over time per household
  #	    hhPropInfectedMean: mean proportion infected per household size
  #	    hhDurationMean:	mean time from first onset to the last case per household size
  
  ## GLOBAL LEVEL
  
  # Status and incidence 
  statusMat <- result$statusMat
  incidenceMat <- result$incidenceMat
  
  # Final size of epidemic
  HH_network <- result$HH_network
  nrVertices <- network.size(HH_network)
  finalSize <- sum(unlist(lapply(incidenceMat, function(x) sum(x))))
  finalFraction <- finalSize/nrVertices
  
  # Dummy variable indicating whether a person has been infected during the epidemic
  infected <- unlist(lapply(incidenceMat, function(x) {apply(x,1,sum)}))
  
  # Duration of infectious period (NA in case the individual did not get infected)
  infPeriod <- unlist(lapply(statusMat, function(x) {apply(x==1,1,sum)}))
  infPeriod[infected == 0] <- NA
  
  
  ## HOUSEHOLD LEVEL
  
  # Extract hh_id and hh_size
  hh_id <- HH_network %v% "hh_id"
  hh_id_unique <- unique(hh_id)
  nrHH <- length(hh_id_unique)
  hh_role <- HH_network %v% "hh_role"
    
  # Incidence of households over time
  hh_incidence <- lapply(incidenceMat, function(x){apply(x,2,sum)})
  # Proportion infected in each household
  hhInfected <- unlist(lapply(incidenceMat, function(x) sum(x)))
  hh_size <- unlist(lapply(incidenceMat, function(x) dim(x)[1]))
  hhPropInfected <- hhInfected/hh_size
  hhSecInfected <- hhInfected[hhInfected!=1]/hh_size[hhInfected!=1]
  ## Number and proportion of households with more than 1 infection
  hhFinalSize <- sum(hhInfected>1)
  hhFinalFraction <- hhFinalSize/nrHH
  # Time from first onset to the last case
  hh_infection_times <- lapply(hh_incidence, function(x) which(x != 0))
  hhDuration <- unlist(lapply(hh_infection_times, function(x){x[length(x)]-x[1]}))
  # Role of index case in each hh
  hhIndexCase <- rep(NA,length(hh_id_unique))
  for (i in 1:length(hh_id_unique)){
    hhComposition <- NA
    hhComposition <- hh_role[hh_id == hh_id_unique[i]]
    hhIndexCase[i] <- hhComposition[incidenceMat[[i]][, min(hh_infection_times[[i]])] == 1][1]  # first element of index case vector
  }
  
  # Mean proportion infected and mean time from first onset to the last case per role index case
  hhcomp <- unique(hh_role)
  hhPropInfected_role <- NULL
  hhDuration_role <- NULL
  for (i in 1:length(hhcomp)){
    hhPropInfected_role[i] <- mean(hhPropInfected[hhIndexCase == hhcomp[i]])
    hhDuration_role[i] <- mean(hhDuration[hhDuration!= 0 & hhIndexCase == hhcomp[i]])
  }
  
  # Mean proportion infected and mean time from first onset to the last case per HH size
  range <- c("<=3","4","5",">=6")
  hhSize_factor <- cut(hh_size, c(2,4:6,13), labels = range, right = FALSE)
  hhPropInfectedMean <- NULL
  hhSecInfectedMean <- NULL
  hhDurationMean <- NULL
  for (i in 1:length(range)){
    hhPropInfectedMean[i] <- mean(hhPropInfected[hhSize_factor == range[i]])
    hhSecInfectedMean[i] <- mean(hhPropInfected[hhInfected!=1 & hhSize_factor == range[i]])
    hhDurationMean[i] <- mean(hhDuration[hhDuration!= 0 & hhSize_factor == range[i]])
  }
  
  ## LIST OF OUTPUT VARIABLES
  output <- list(finalSize = finalSize, finalFraction = finalFraction, infPeriod = infPeriod, hhSize = hh_size, hhFinalSize = hhFinalSize, hhFinalFraction = hhFinalFraction
                 , hhIncidence = hh_incidence, hhPropInfected = hhPropInfected, hhSecInfected = hhSecInfected, hhDuration = hhDuration, hhIndexCase = hhIndexCase
                 , hhPropInfectedMean = hhPropInfectedMean, hhSecInfectedMean = hhSecInfectedMean, hhDurationMean = hhDurationMean, hhDuration_role = hhDuration_role, hhPropInfected_role = hhPropInfected_role)
  
  ## PRINT OUTPUT VARIABLES
  if (display == TRUE){
    print("Final size")	
    print(output[c(1, 2, 5, 6)])
    
    print("Summary latent period")
    print(summary(latentPeriod))
    
    print("Mean proportion infected per household size")
    temp <- sort(unique(hh_size))
    for (i in 1:length(temp)){
      print(paste("Household size ", temp[i], sep =""))
      print(hhPropInfectedMean[i])
    }
    
    print("Mean time from first onset to the last case per household size")
    for (i in 1:length(temp)){
      print(paste("Household size ", temp[i], sep =""))
      print(hhDurationMean[i])
    }
  }
  
  ## RETURN OUTPUT
  return(output)
}







